@@ -1,7 +1,7 @@
 import os
 import re
 import urllib
-from ConfigParser import RawConfigParser
+from ConfigParser import ConfigParser
 import pkg_resources
 
 __all__ = ['loadapp', 'loadserver', 'loadfilter']
@@ -36,6 +36,23 @@ def _flatten(lst):
         result.extend(_flatten(item))
     return result
 
+class NicerConfigParser(ConfigParser):
+
+    def __init__(self, filename, *args, **kw):
+        ConfigParser.__init__(self, *args, **kw)
+        self.filename = filename
+
+    def _interpolate(self, section, option, rawval, vars):
+        try:
+            return ConfigParser._interpolate(
+                self, section, option, rawval, vars)
+        except Exception, e:
+            args = list(e.args)
+            args[0] = 'Error in file %s, [%s] %s=%r: %s' % (
+                self.filename, section, option, rawval)
+            e.args = tuple(args)
+            raise
+
 ############################################################
 ## Object types
 ############################################################
@@ -229,11 +246,11 @@ class _Loader(object):
 
     def get_filter(self, name=None, global_conf=None):
         return self.filter_context(
-            name=naame, global_conf=global_conf).create()
+            name=name, global_conf=global_conf).create()
 
     def get_server(self, name=None, global_conf=None):
         return self.server_context(
-            name=naame, global_conf=global_conf).create()
+            name=name, global_conf=global_conf).create()
 
     def app_context(self, name=None, global_conf=None):
         return self.get_context(
@@ -260,7 +277,7 @@ class ConfigLoader(_Loader):
 
     def __init__(self, filename):
         self.filename = filename
-        self.parser = RawConfigParser()
+        self.parser = NicerConfigParser(self.filename)
         # Don't lower-case keys:
         self.parser.optionxform = str
         # Stupid ConfigParser ignores files that aren't found, so
@@ -269,6 +286,8 @@ class ConfigLoader(_Loader):
             raise OSError(
                 "File %s not found" % filename)
         self.parser.read(filename)
+        self.parser._defaults.setdefault(
+            'here', os.path.dirname(os.path.abspath(filename)))
 
     def get_context(self, object_type, name=None, global_conf=None):
         if self.absolute_name(name):
@@ -285,20 +304,30 @@ class ConfigLoader(_Loader):
         global_conf.update(defaults)
         local_conf = {}
         global_additions = {}
+        get_from_globals = {}
         for option in self.parser.options(section):
             if option.startswith('set '):
                 name = option[4:].strip()
                 global_additions[name] = global_conf[name] = (
                     self.parser.get(section, option))
+            elif option.startswith('get '):
+                name = option[4:].strip()
+                get_from_globals[name] = self.parser.get(section, option)
             else:
                 if option in defaults:
                     # @@: It's a global option (?), so skip it
                     continue
                 local_conf[option] = self.parser.get(section, option)
+        for local_var, glob_var in get_from_globals.items():
+            local_conf[local_var] = global_conf[glob_var]
         if object_type is APP and 'filter-with' in local_conf:
             filter_with = local_conf.pop('filter-with')
         else:
             filter_with = None
+        if 'require' in local_conf:
+            for spec in local_conf['require'].split():
+                pkg_resources.require(spec)
+            del local_conf['require']
         if section.startswith('filter-app:'):
             context = self._filter_app_context(
                 object_type, section, name=name,
@@ -482,14 +511,20 @@ class EggLoader(_Loader):
                     break
         if not possible:
             # Better exception
+            dist = pkg_resources.get_distribution(self.spec)
             raise LookupError(
-                "Entry point %r not found in egg %r (protocols: %s)"
+                "Entry point %r not found in egg %r (dir: %s; protocols: %s; "
+                "entry_points: %s)"
                 % (name, self.spec,
-                   ', '.join(_flatten(object_type.egg_protocols))))
+                   dist.location,
+                   ', '.join(_flatten(object_type.egg_protocols)),
+                   ', '.join(_flatten([
+                (pkg_resources.get_entry_info(self.spec, prot, name) or {}).keys()
+                for prot in protocol_options] or '(no entry points)'))))
         if len(possible) > 1:
             raise LookupError(
                 "Ambiguous entry points for %r in egg %r (protocols: %s)"
-                % (name, self.spec, ', '.join(_flatten(protocol_list))))
+                % (name, self.spec, ', '.join(_flatten(protocol_options))))
         return possible[0]
 
 class LoaderContext(object):
